/* BASE64 encoding encodes 3 bytes into 4 characters.
   |11111122|22223333|33444444|
   Each set of 6 bits is encoded according to the 
   toBase64 map. If the number of input bytes is not
   a multiple of 3, then the last group of 4 characters
   is padded with one or two = signs. Each output line
   is at most 76 characters.
*/

import java.io.*;

class Base64OutputStream extends OutputStream
{  public Base64OutputStream(StringBuffer sb)
   {  sbuf = sb;
   }

   public void write(int c) throws IOException
   {  inbuf[i] = c;
      i++;
      if (i == 3)
      {  sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
         sbuf.append(toBase64[((inbuf[0] & 0x03) << 4) |
            ((inbuf[1] & 0xF0) >> 4)]);
         sbuf.append(toBase64[((inbuf[1] & 0x0F) << 2) |
            ((inbuf[2] & 0xC0) >> 6)]);
         sbuf.append(toBase64[inbuf[2] & 0x3F]);   
         col += 4;
         i = 0;
         if (col >= 76)
         {  sbuf.append('\n');
            col = 0;
         }
      }
   }

   public void flush()
   {  if (i == 1)
      {  sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
         sbuf.append(toBase64[(inbuf[0] & 0x03) << 4]);
         sbuf.append('=');
         sbuf.append('=');
      }
      else if (i == 2)
      {  sbuf.append(toBase64[(inbuf[0] & 0xFC) >> 2]);
         sbuf.append(toBase64[((inbuf[0] & 0x03) << 4) |
            ((inbuf[1] & 0xF0) >> 4)]);
         sbuf.append(toBase64[(inbuf[1] & 0x0F) << 2]);
         sbuf.append('=');
      }
      sbuf.append('\n');
   }

   private static char[] toBase64 =
   {  'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
      'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P',
      'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X',
      'Y', 'Z', 'a', 'b', 'c', 'd', 'e', 'f',
      'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
      'o', 'p', 'q', 'r', 's', 't', 'u', 'v',
      'w', 'x', 'y', 'z', '0', '1', '2', '3',
      '4', '5', '6', '7', '8', '9', '+', '/'
   };
   
   StringBuffer sbuf;
   int col = 0;
   int i = 0;
   int[] inbuf = new int[3];
}

class Base64InputStream extends InputStream
{  public Base64InputStream(String s, int start)
   {  str = s;
      pos = start;
      i = 0;
   }

   public int read() 
   {  while (pos < str.length() && 
         Character.isWhitespace(str.charAt(pos)))
      {  pos++;
      }
      if (pos >= str.length()) return -1;
      if (i == 0)
      {  int ch1 = str.charAt(pos) & 0x7F;
         int ch2 = str.charAt(pos + 1) & 0x7F;
         i++;
         return (fromBase64[ch1] << 2)
            | (fromBase64[ch2] >> 4);
      }
      else if (i == 1)
      {  int ch1 = str.charAt(pos + 1) & 0x7F;
         int ch2 = str.charAt(pos + 2) & 0x7F;
         if (ch2 == '=') return -1;
         i++;
         return ((fromBase64[ch1] & 0x0F) << 4)
            | (fromBase64[ch2] >> 2);
      }
      else
      {  int ch1 = str.charAt(pos + 2) & 0x7F;
         int ch2 = str.charAt(pos + 3) & 0x7F;
         if (ch2 == '=') return -1;
         i = 0;
         pos += 4;
         return ((fromBase64[ch1] & 0x03) << 6)
            | fromBase64[ch2];
      }
   }

   private static int[] fromBase64 =
   {  -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, -1, -1, -1, -1, -1,
      -1, -1, -1, 62, -1, -1, -1, 63,
      52, 53, 54, 55, 56, 57, 58, 59,
      60, 61, -1, -1, -1, -1, -1, -1,
      -1,  0,  1,  2,  3,  4,  5,  6,
       7,  8,  9, 10, 11, 12, 13, 14,
      15, 16, 17, 18, 19, 20, 21, 22,
      23, 24, 25, -1, -1, -1, -1, -1,
      -1, 26, 27, 28, 29, 30, 31, 32,
      33, 34, 35, 36, 37, 38, 39, 40,
      41, 42, 43, 44, 45, 46, 47, 48,
      49, 50, 51, -1, -1, -1, -1, -1
   };

   String str;
   int pos;
   int i;
}
