/*
 *  FILE    : Base64.java
 *  Ver     : 1.1      (July, `99)
 *  AUHTHOR : Sangyoon Oh
 *            soh@napc.syr.edu
 *  
 *  This class encode image file to readable text.
 *  Public method encode will do base 64 encode.
 *  Public method send will send out to PrintWriter.
 *
 *  API : public static void encode(byte[] bytes, PrintWriter out)
 *        public static void send(PrintWriter out, String s)
 *
 *
 *  Maintenance :
 *      ver 1.0 : created Base64 encoding Module
 *      ver 1.1 : Minor change on Byte array input to encode()
 *
 *
 */

import java.io.*;

public class Base64 {
 
    // encoding module
    public static void encode(byte[] bytes, PrintWriter out) {
        try {
	    // get the length of byte array	
	    int n = bytes.length;
	
            // check file existence
            if (n < 1) return;     
            
            // get three bytes from file and encode it into four 
            byte buf[] = new byte[4];   
            
            // how many group of three bytes
            int ndiv = n / 3;
            
            // for padding ( = or == ) at the end 
            int mod = n % 3;     
            
            // how many times for looping
            int k = ndiv * 3; 
            
            int length = 0;         
            int i = 0;    

            // do base64 encoding for k times
            // encoding using mask and shift
            while ( i < k ) {
                buf[0] = (byte)(( bytes[i]   & 0xFC) >> 2); 
                buf[1] = (byte)(((bytes[i]   & 0x03) << 4) | 
                  ((bytes[i+1] & 0xF0) >> 4));
                buf[2] = (byte)(((bytes[i+1] & 0x0F) << 2) | 
                  ((bytes[i+2] & 0xC0) >> 6));
                buf[3] = (byte)(bytes[i+2] & 0x3F);
                send(out, toBase64[buf[0]]);
                send(out, toBase64[buf[1]]);
                send(out, toBase64[buf[2]]);
                send(out, toBase64[buf[3]]);
           
                // line should be less than 76 character
                if ((length += 4) >= 76) {
                    send(out, "\r\n");
                    length = 0;
                }
                i += 3;
            }

            // padding at the end
            if (mod==2) {
                buf[0] = (byte)((bytes[k] & 0xFC)   >> 2);
                buf[1] = (byte)(((bytes[k] & 0x03)   << 4) | 
                   ((bytes[k+1] & 0xF0) >> 4));
                buf[2] = (byte)((bytes[k+1] & 0x0F) << 2); 
                send(out, toBase64[buf[0]]);
                send(out, toBase64[buf[1]]);
                send(out, toBase64[buf[2]]);
                send(out, "=");
            } 
            else if (mod==1) {
                buf[0] = (byte)((bytes[k] & 0xFC) >> 2);
                buf[1] = (byte)((bytes[k] & 0x03) << 4);
                send(out, toBase64[buf[0]]);
                send(out, toBase64[buf[1]]);
                send(out, "=");
                send(out, "=");
            }
            send(out, "\r\n");
            out.flush();
        } 
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    // sendout method with PrintWriter
    public static void send(PrintWriter out, String s) {
        try {
            out.write(s);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    // Base64 encoding char array
    static String toBase64[] = { 
        "A","B","C","D","E","F","G","H",
        "I","J","K","L","M","N","O","P",
        "Q","R","S","T","U","V","W","X",
        "Y","Z","a","b","c","d","e","f",
        "g","h","i","j","k","l","m","n",
        "o","p","q","r","s","t","u","v",
        "w","x","y","z","0","1","2","3",
        "4","5","6","7","8","9","+","/"
    };
}





