/*
 *  File   : MailServerImpl.java
 *  ver    : 1.1      (July, `99)
 *  Author : Sangyoon Oh
 *           soh@npac.syr.edu
 *
 *  This MailServerImpl class provide implementation of Server object
 *
 *  API : public MailServerImpl() throws RemoteException
 *        public void sendEMail(String sender, 
 *                     String senderAdr, Vector toRcpts, Vector ccRcpts, 
 *                     String subject, String context)
 *                     throws java.rmi.RemoteException  
 *        public void sendEMail(String sender, 
 *                     String senderAdr, Vector toRcpts, Vector ccRcpts, 
 *                     String subject, String context, byte[] bytes, 
 *                     String fType, String fName)
 *                     throws java.rmi.RemoteException 
 *
 *
 *  Maintenance :
 *      ver 1.0 : created MailServer application Module
 *      ver 1.1 : minor change on parameter (smtphost)
 *      ver 1.2 : RMI registry object is created in main()
 *
 */

import java.rmi.*;
import java.rmi.server.*;
import java.rmi.registry.*;
import java.io.*;
import java.util.*;
import java.util.Date;
import java.sql.*;
import webwisdom.tango.newca.view.MailServer;

public class MailServerImpl extends UnicastRemoteObject implements MailServer {

    // ctor
    public MailServerImpl() throws RemoteException {
	super();
    }

    // implementation of sendEmail() which is without Attachment File
    public void sendEMail(String sender, String senderAdr, Vector toRcpts, Vector ccRcpts, String subject, String context) 
	throws java.rmi.RemoteException {
	try {

	    // set parameters
	    tm = new TangoMail(smtpHost);
	    tm.setFrom(sender, senderAdr);
	    tm.addTo(toRcpts);
	    tm.addCc(ccRcpts);
	    tm.setSubject(subject);
	    tm.setContext(context);
	    tm.sendMail();
	}

	// should catch MailException
	catch( MailException e ) {
	    System.out.println("Error : " + e.getMessage());
	}
    }
        
    // implementation of sendEmail() which is with Attachment File
    public void sendEMail(String sender, String senderAdr, Vector toRcpts, Vector ccRcpts, String subject, String context, byte[] bytes, String fType, String fName)                
	throws java.rmi.RemoteException {
	try {

	    // set parameters
	    tm = new TangoMail(smtpHost);
	    tm.setFrom(sender, senderAdr);
	    tm.addTo(toRcpts);
	    tm.addCc(ccRcpts);
	    tm.setSubject(subject);
	    tm.setContext(context);
	    tm.setAttachment(bytes, fType, fName);
	    tm.sendMail();
	}
	// should catch MailException
	catch( MailException e ) {
	    System.out.println("Error : " + e.getMessage());
	}
    }

    // main()
    public static void main(String[] args) {
	try {
	    // Set the standard RMI security manager for the stub
	    System.setSecurityManager(new RMISecurityManager());
	    
	    // Create a server object
	    MailServerImpl obj = new MailServerImpl();
	    
	    // register the server
	    Registry registry = LocateRegistry.createRegistry(1099); 
	    registry.bind("MailServer", obj); 

	    // setting smtphost name
	    smtpHost = args[0];
	}
	catch (Exception e) {
	    System.err.println("MailServer Exception : " + e);
	}
    }
    
    // TangoMail class object
    private TangoMail tm;

    // smtphost name
    private static String smtpHost = "localHost";
}







