/*
 *  FILE    : TangoMail.java
 *  Ver     : 1.2      (July, `99)
 *  AUHTHOR : Sangyoon Oh
 *            soh@napc.syr.edu
 *  
 *  This class provide SMTP/MIME mail function.
 *  Public method will user interface for setting parameters.
 *
 *  API     : public TangoMail(String host)
 *            public TangoMail()
 *            public void setFrom (String realName, String address)
 *            public void setDebug (boolean how)
 *            public void addTo (Vector toRcpts)
 *            public void addCc (Vector ccRcpts)
 *            public void setContext (String text)
 *            public void setSubject( String subject ) 
 *            public void setAttachment( byte[] source, 
 *                                       String fType, String fName )
 *
 *  Maintenance :
 *      ver 1.0 : Implementation of SMTP Protocol
 *      ver 1.1 : Implementation of SMTP/MIME Protocol
 *      ver 1.2 : Minor change for Attachment File Setting
 *                Byte array input rather than File Object to Base64 class 
 *
 *
 */


import java.lang.*;
import java.util.*;
import java.net.*;
import java.io.*;

public class TangoMail {

    // initiate 
    private void initVar() {
	toRecipients.put("toname","unknown");
	toRecipients.put("toadr","root@localhost"); 
	ccRecipients.put("ccname","unknown");
	ccRecipients.put("ccadr","root@localhost");
            
	toVector.addElement(toRecipients);
	ccVector.addElement(ccRecipients);
    }
    
    // ctor -- specific host  
    public TangoMail(String host) {
 	smtpHost = host;
	initVar();              		
    }
  
    // ctor -- local host   
    public TangoMail() {
	initVar();
    }
    
    // setfrom  
    public void setFrom (String realName, String address) {
	fromName = realName ;
	fromAdr = address;
    }

    // setDebug
    public void setDebug (boolean how) {
	doDebug = how ;
    }
    
    // get toVector 
    public void addTo (Vector toRcpts) {
	toVector = toRcpts;            
    }

    // get ccVector 
    public void addCc (Vector ccRcpts) { 
	ccVector = ccRcpts;
    }

    // context
    public void setContext (String text) {
	sText.append(text + "\n");
    }
     
    // setSubject 
    public void setSubject( String subject ) {
	sSubject = subject ;
    }
    
    // set Attachment
    public void setAttachment( byte[] source, String fType, String fName ) {
	bytes = source;
	fileType = fType;
	fileName = fName;
    }

    // sendMail       
    public void sendMail() throws MailException  {
        try {
            int i;
            
            try {  
                s = new Socket(smtpHost,25);
            }
            catch (Exception e) {
                System.out.println("Exception " +e);
            }

            pout = new PrintWriter(s.getOutputStream(),true);
            pin = new DataInputStream(s.getInputStream());
            checkResponse(220);
              
            // SMTP
            sendOut("HELO " + authName ,250);  
            sendOut("MAIL FROM: <" + fromAdr + ">", 250);

	     // target adressees
            for( i = 0 ; i < toVector.size(); i++) { 
                toRecipients = (Hashtable) toVector.elementAt(i);
                toAdr  = (String) toRecipients.get("toadr");   
                sendOut("RCPT TO: <" + toAdr + ">", 250);
            }
            String sCcAdr=null;
            if (ccVector!=null && !ccVector.isEmpty() ) {
	      Hashtable ht=(Hashtable) ccVector.elementAt(0);

	      if (ht!=null) {
                sCcAdr = (String)((Hashtable) ccVector.elementAt(0)).get("ccadr");

		if  ((sCcAdr.compareTo("root@localhost")!= 0)&&(sCcAdr.length()!=0)) {
		  for ( i = 0; i < ccVector.size() ; i++) { //cc
		    ccRecipients = (Hashtable) ccVector.elementAt(i);
		    ccAdr  = (String) ccRecipients.get("ccadr");  
		    sendOut("RCPT TO: <" + ccAdr + ">", 250);
		  }
		}
	      }
            }
                        
            // Data
            sendOut("DATA", 354);
            if (doDebug == true) { 
		System.out.println("<--- SMTPmail Debug : Sending DATA Part --->");
            }    
              
            // STD Fields     
            pout.println("DATE: " + new Date(System.currentTimeMillis()));
            pout.println("From: " + standardField(fromName, fromAdr)); // from
            pout.println("Sender: " + standardField(fromName, fromAdr)); // Sender
            
            // To
            for ( i = 0; i < toVector.size() ; i++) { //to
                toRecipients = (Hashtable) toVector.elementAt(i);
                toName  = (String) toRecipients.get("toname");  
                toAdr  = (String) toRecipients.get("toadr");  
		pout.println("To: " + standardField(toName, toAdr));
            }

            // Cc            
            if (ccVector!=null && !ccVector.isEmpty() ) {
	      if (sCcAdr!=null) {

                if  ((sCcAdr.compareTo("root@localhost")!=0)&&(sCcAdr.length()!=0)) { 
		  for ( i = 0; i < ccVector.size() ; i++) { //cc
		    ccRecipients = (Hashtable) ccVector.elementAt(i);
		    ccName  = (String) ccRecipients.get("ccname");  
		    ccAdr  = (String) ccRecipients.get("ccadr");  
		    pout.println("Cc: " + standardField(ccName, ccAdr));  
		  }
                }
	      }
            }

            // Reply-To
            if  ( fromName.compareTo("unknown") == 0 )  {}
	    else {
                    pout.println("Reply-To: " + standardField(fromName, fromAdr));
	    }          
	        
	    // Subject
            pout.println("Subject: " + sSubject);
	                
	    // MIME Multi-part
	    pout.println("MIME-Version:1.0");
    	    pout.println("Content-Type: Multipart/mixed; boundary=\"sb\"");
    	    pout.println("--sb");
    	    pout.println("");
         
            // Context 
            if ( sText.length() > 0 ) {
	            pout.println(sText);
            }
		
            if(fileName.compareTo("unknown")!=0) {
		pout.println("--sb");
		pout.println("Content-Type: " + fileType + "; name=" + fileName);
		pout.println("Content-Disposition: attachment; filename=\"" + fileName + "\"");
		pout.println("Content-transfer-encoding: base64");	
		pout.println("");
	    
		// do base64 encode
		mb = new Base64();
		mb.encode(bytes, pout);
		pout.println("");
            }

            // MIME end
            pout.println("");
            pout.println("--sb--"); 
            pout.println("");
            
            // done
            pout.println(".");     
            checkResponse(250);
            System.out.println("Mail sent to recipients.");
        } 
         
        catch (IOException e) {
            if ( doDebug == true) {
		System.out.println("<--- SMTPmail Debug : Error " + e.getMessage() + " throw exception." );
            }   
            throw (new MailException("IOException collected: " + e.getMessage()));
        }

        // try to quit protocol
        try {
            sendOut("QUIT", 221);
            s.close();
        } 
	catch (IOException e1) {  // do nothing  
	}        
        
        if ( doDebug == true) {
            System.out.println("<--- SMTPmail Debug : done --->");
        }   
    }
    
    // building standard field for SMTP protocol
    private String standardField( String cText, String address) {    
	return (cText + " <" + address + "> ");
    }
      
    // send SMTP Protocol
    private void sendOut ( String msg, int respvalue ) throws MailException {
      
        // read and write channel are global
        pout.println(msg);	
        if (doVerify == true) {
            if (doDebug == true) { 
		System.out.println("<--- SMTPmail Debug : Sent \n" + msg);    
            }
            checkResponse( respvalue );
        }
    }    
        
    private StringBuffer readLine ( DataInputStream fin ) throws IOException {    
	// read one line get File in Stream
	StringBuffer line = new StringBuffer(""); 
	char b;   
	
	while (fin.available() > 0) {
	    b = (char) fin.read();
	    if ( b == '\n' ) {
		// one line read
		break;
	    }
	    line.append(b);
	}
	return (line);
    }
    
    private void checkResponse (int respvalue) throws MailException {
	
	String response = "";
	int counter = 0;
	while ( response.length() == 0 ) {
	    try { Thread.sleep(200);
	    } 
	    catch ( InterruptedException e1) {  // do nothing
	    }
	    
	    try {
		response = new String(readLine(pin));
	    } 
	    catch ( IOException e ) {
		throw (new MailException("IOException while reading from smtpd : " + e.getMessage()));
	    }
	    
	    counter += 1;
	    if (counter > timeOut) {
		// try to stop: somthing wrong
		throw (new MailException("SMTPmail : Timeout while waiting for smtpd response."));	
	    }
	}
                
	if (doDebug == true) { 
	    System.out.println("<--- SMTPmail Debug : Response \n" + response );
	}
	
	// check return value
	int retval = 0;
	try {
	    retval =   ((int)  Integer.parseInt( response.substring(0,3)) );
	} 
	catch (Exception e) {
	    retval = 0;
	}
                
	if ( retval == respvalue ) {
	    if (doDebug == true) { 
		System.out.println("<--- SMTPmail Debug : Response Ok (" + retval + ")---> ");
	    }
	} 
	else {
	    if (doDebug == true) { 
		System.out.println("<--- SMTPmail Debug : Response NOT Ok (Should be " + response + " but is " + retval + "Troughd exception." );
	    }
	    throw (new MailException("Wrong smtpd response: Should be " + response + ", is " + retval ));
	}
    }
    
    // global vars
    // String
    private String smtpHost = "localhost" ;
    private String fromName = "unknown";
    private String fromAdr = "unknown@localhost";
    private String toName = "unknown";
    private String toAdr = "unknown@localhost";
    private String ccName = "unknown";
    private String ccAdr = "unknown@localhost";   
    private String sSubject;
    private String authName = "TangoMail";
    private String fileName = "unknown";
    private String fileType;
    
    // vector
    private Vector toVector = new Vector();
    private Vector ccVector = new Vector();

    // byte array
    private byte bytes[];	
    private int n;
    
    // hashtable
    private Hashtable toRecipients = new Hashtable();
    private Hashtable ccRecipients = new Hashtable();

    // text
    private StringBuffer sText = new StringBuffer();
    
    // debug and verify
    private boolean doDebug = false;
    private static boolean doVerify = true;
    private static int timeOut = 20;           //is time times 200 millies  

    // io 
    Socket s = null;
    PrintWriter pout;
    DataInputStream pin;

    // Base64 class object
    private Base64 mb;
}

